function KL = KLdivergence(p,q)
% KL = KLdivergence(p,q)
%
% Compute KL divergence: sum_i p_i*(log(p_i)-log(q_i))
%
% Inputs:
% p - LxM matrix of probabilities, stored as column vectors.
% q - LxN matrix of probabilities, stored as column vectors.
%
% Outputs:
% KL - MxN matrix of KL divergences.

% Normalize p and q so that the columns sum to one:
[i,j] = find(p);
n = find(p);
pSum = sum(p,1)';
p(n) = p(n)./pSum(j);
%p = p./repmat(sum(p,1),size(p,1),1);
q = q./repmat(sum(q,1),size(q,1),1);

% Compute KL divergence:
plogp = p;
plogp(n) = p(n).*log(p(n));
KL = repmat(sum(plogp,1)',1,size(q,2)) - p'*log(q);
