% demoLabelMe.m
% Demonstration of the object discovery code on LabelMe images.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Modify the following lines to point to downloaded Matlab libraries,
% binaries, visual word clusters, LabelMe images and annotations, and
% desired locations of outputs.

% Matlab libraries.  Change these lines to point to the location of the
% installed Matlab libraries (LabelMe toolbox and Ncuts).
addpath /path/to/LabelMeToolbox;
addpath /path/to/Ncut_9;

% Global variables.  Change these to point to the location of the
% downloaded Oxford visual word binaries, Oxford visual word clusters,
% and LabelMe images and annotations.
HOMEIMAGES = '/path/to/Images'; % LabelMe images
HOMEANNOTATIONS = '/path/to/Annotations'; % LabelMe annotations
OXFORD_VISUAL_WORD_BINARIES = '/path/to/OxfordVisualWordBinaries'; % OxfordVisualWordBinaries
OXFORD_VISUAL_WORD_CLUSTERS = '/path/to/OxfordVisualWordClusters'; % OxfordVisualWordClusters

% Output directories.  Change these to point to where you would like
% computed visual words, segments, and visual word->segment assignments
% to live.
HOMEWORDS = '/path/to/VisualWordsOutputs';
HOMENCUTS = '/path/to/NcutsOutputs';
HOMEMERGE = '/path/to/MergeOutputs';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters:
Ntopics = 30; % Number of topics to discover


HARRIS_AFFINE_BIN = fullfile(OXFORD_VISUAL_WORD_BINARIES,'detect_points.ln');
MSER_BIN = fullfile(OXFORD_VISUAL_WORD_BINARIES,'mser.ln');
DESCRIPTOR_BIN = fullfile(OXFORD_VISUAL_WORD_BINARIES,'compute_descriptors.ln');
clusterFilenames = {fullfile(OXFORD_VISUAL_WORD_CLUSTERS,'d101-tp372072_n1_sift3_s1.0_pp90_K01086_t0.VQadam'),fullfile(OXFORD_VISUAL_WORD_CLUSTERS,'d101-tp384813_n1_sift3_s1.0_pp90_K01138_t0.VQjiri')};

% Load LabelMe images:
DB = LMdatabase(HOMEANNOTATIONS);

% Load SIFT clusters:
[Clusters,numCenters] = loadSiftClusters(clusterFilenames);
V = sum(numCenters);

% Accumulate features and segments:
wordsDocuments = zeros(V,length(DB)*100);
imageNdx = zeros(1,length(DB)*100);
segmentNdx = zeros(1,length(DB)*100,'uint16');
j = 0;
for i = 1:length(DB)
  display(sprintf('%d out of %d',i,length(DB)));
  
  % Read and process image:
  img = LMimread(DB,i,HOMEIMAGES);
  [interestPoints,visualWords,regionTypes,angles] = computeVisualWords(img,Clusters,numCenters,HARRIS_AFFINE_BIN,MSER_BIN,DESCRIPTOR_BIN);
  Segs = multipleSegmentations(img);
  merge = mergeWordsSegments(interestPoints,Segs);
  F = wordFrequencyMatrix(V,visualWords,merge);
  wordsDocuments(:,j+1:j+size(F,2)) = sparse(F);
  imageNdx(j+1:j+size(F,2)) = i;
  segmentNdx(j+1:j+size(F,2)) = [1:size(F,2)];
  j = j+size(F,2);
  
  % Write outputs for later:
  if ~exist(fullfile(HOMEWORDS,DB(i).annotation.folder),'dir')
    mkdir(fullfile(HOMEWORDS,DB(i).annotation.folder));
  end
  if ~exist(fullfile(HOMENCUTS,DB(i).annotation.folder),'dir')
    mkdir(fullfile(HOMENCUTS,DB(i).annotation.folder));
  end
  if ~exist(fullfile(HOMEMERGE,DB(i).annotation.folder),'dir')
    mkdir(fullfile(HOMEMERGE,DB(i).annotation.folder));
  end
  save(fullfile(HOMEWORDS,DB(i).annotation.folder,[DB(i).annotation.filename '.mat']),'interestPoints','visualWords','regionTypes','angles');
  save(fullfile(HOMENCUTS,DB(i).annotation.folder,[DB(i).annotation.filename '.mat']),'Segs');
  save(fullfile(HOMEMERGE,DB(i).annotation.folder,[DB(i).annotation.filename '.mat']),'merge');
end
wordsDocuments = wordsDocuments(:,1:j);
imageNdx = imageNdx(1:j);
segmentNdx = segmentNdx(1:j);

% Perform clustering and analysis:
[pWordsTopics,pTopicsDocuments] = lda(wordsDocuments,Ntopics);
KL = KLdivergence(wordsDocuments,pWordsTopics);

% Display the top segments (as a montage) belonging to a discovered
% topic:
NsegmentsToDisplay = 25;
for i = 1:Ntopics
  n = sortTopicSegments(KL(:,i),imageNdx);
  n = n(1:NsegmentsToDisplay);
  M = topicMontage(DB,imageNdx(n),segmentNdx(n),HOMEIMAGES,HOMENCUTS,HOMEMERGE);
  imshow(M);
  title(i);
  display('Press enter');
  pause;
end
