function [Clusters,numCenters] = loadSiftClusters(clusterFilenames)
% [Clusters,numCenters] = loadSiftClusters(clusterFilenames)
%
% Load pre-computed SIFT descriptor cluster centers.
%
% Inputs:
% clusterFilenames - Cell array of paths to Matlab MAT files containing
%   pre-computed SIFT cluster centers.  Each entry in the cell array
%   corresponds to a set of clusters computed over a particular interest
%   region type (e.g. Harris affine, MSER).
%
% Outputs:
% Clusters - Cell array of cluster structures.
% numCenters - Number of clusters corresponding to each structure.

N = length(clusterFilenames);
for i = 1:N
  Clusters{i} = getfield(load(clusterFilenames{i},'-mat'),'Cl');
  numCenters(i) = size(Clusters{i}.Centers,1);
end
