function merge = mergeWordsSegments(interestPoints,Segs)
% merge = mergeWordsSegments(interestPoints,Segs)
%
% Merge visual words and segments (i.e. determine which visual words
% intersect each segment).
%
% Inputs:
% interestPoints - Interest points.
% Segs - (Multiple) segmentations.
%
% Outputs:
% merge - Structure array containing information about which visual words
%   intersect which segment.  The structure looks as follows:
%
%   For segment i:
%   merge(i).wordsNdx - Indices to the visual words that lie in the segment.
%   merge(i).segmentationNdx - Indexes which segmentation mask.
%   merge(i).segmentNdx - Indexes the segment within the segmentation mask.

merge = [];
k = 0;
for i = 1:size(Segs,3)
  segmentation = Segs(:,:,i);
  Nsegments = max(segmentation(:));
  for j = 1:Nsegments
    % Find overlapping visual words:
    x = floor(interestPoints(1,:));
    y = floor(interestPoints(2,:));
    n = find(segmentation(sub2ind(size(segmentation),y,x))==j);
    
    % If there are overlapping visual words, then store into "merge"
    % structure:
    if ~isempty(n)
      k = k+1;
      merge(k).wordsNdx = uint16(n);
      merge(k).segmentationNdx = uint8(i);
      merge(k).segmentNdx = uint8(j);
    end
  end
end
