function [ellh, helc,XX,YY] = plotInterestPoints(Im, fts, s_region, ellcol, ...
                                                 figattr, thickness);
% plotInterestPoints(img,interestPoints)
% Plots ellipses around detected interest points to indicate their region
% of support.
%
% Inputs:
% img - Input image
% interestPoints - Detected interest points.
%
% Josef Sivic <josef@robots.ox.ac.uk>
% 18/09/2002
% Modified by:
% Bryan Russell <brussell@csail.mit.edu>

if nargin < 3
    s_region = 5;  %s * unit_disk in the normalized frame    
    s_region = 1;
end;

if size(s_region,1) ==1 
   s_region = s_region * ones(size(fts,2),1);
end;

if nargin < 4
    ellcol = 'w';
    ellcol = 'y';
end;

if nargin < 5     
% $$$     figh = figure;
  figh = gcf;
    figattr(2)=1; figattr(3)=1; figattr(4)=1;
else
    figh = figattr(1);      
    if length(figattr)==1 
        figattr(2)=1; figattr(3)=1; figattr(4)=1;
    end;
end;

if nargin < 6
  thickness = 2;
end


% init
ellh = []; helc = [];
%fts
% $$$ if isempty(fts)
% $$$    return;
% $$$ end;   

figure(figh); hs = subplot(figattr(2),figattr(3),figattr(4));

if nargin < 5 % if figure was not specified draw image
  imagesc(Im); 
  hold on; colormap gray; axis equal, axis tight;
end;

% if there is no image in the figure draw one
if isempty(findobj(get(hs,'children'),'type','image'))
  imagesc(Im); 
  hold on; colormap gray; axis equal, axis tight;
else
  % replace the old one with the new one
  fighim =  findobj(hs,'type','image');
  set(fighim,'CData',Im);
end;

if isempty(fts)
% $$$   title('No features to display');
  set(findobj(gcf,'type','axes'),'xtick',[],'ytick',[]);            
  return;
end;

% make unit circle 
npoints = 100; %20; %100;  % number of points on the circle
x = 0;
y = 0;
radius = 1;

theta = [0:(npoints-1) 0] * 2 * pi / npoints;
xx = radius * cos(theta) + x * ones(1,npoints+1);
yy = radius * sin(theta) + y * ones(1,npoints+1);
xx = xx';
yy = yy';


% draw ellipse points
XX = zeros(size(xx,1),size(fts,2));
YY = zeros(size(yy,1),size(fts,2));
%ellh = plot(fts(1,:),fts(2,:),'.w');

for is = 1:size(fts,2)           
   xo = fts(1,is);
   yo = fts(2,is);
   at = fts(3,is);
   bt = fts(4,is);
   ct = fts(5,is);
         
   T = [at bt;bt ct]*s_region(is);

   %sc = sqrt(det(T));
   %Tn = T / sc;
   
   XT = T * [xx'; yy'];
   XX(:,is) = XT(1,:)' + xo;
   YY(:,is) = XT(2,:)' + yo;                  
end;

% plot center of the ellipse
% REMOVE THE COMMENTS IF CENTER POINTS ARE DESIRED:
helc = plot(fts(1,:), fts(2,:), 'r+', 'markersize',5);

% plot all ellipse's points
ellh = plot(XX, YY,'Color', ellcol,'LineWidth',thickness);
% ellh = plot(XX, YY, ellcol,'LineWidth',thickness);
% $$$ ellh = plot(XX, YY, ellcol);

% plot orientation
ellr = [];
if size(fts,1)==6
   xo = fts(1,:);
   yo = fts(2,:);
   at = fts(3,:);
   bt = fts(4,:);
   ct = fts(5,:);
   th = fts(6,:);   
   xxr = cos(th);
   yyr = sin(th);
   
   XXr = (at.*xxr + bt.*yyr).*s_region' + xo;
   YYr = (bt.*xxr + ct.*yyr).*s_region' + yo;   
   %XrT = T * [xxr; yyr];
   %XXr = XrT(1,:)+fts(1,:);
   %YYr = XrT(2,:)+fts(2,:);
   ellr = line([fts(1,:); XXr],[fts(2,:); YYr]);
   set(ellr,'color','w','linewidth',1)
end;   


ellh = [ellh; helc; ellr];
%grid on;

% remoce axis ticks
set(findobj(gcf,'type','axes'),'xtick',[],'ytick',[]);            


%         % how the equation of unit circle transfors with T?
%         syms a b c real
%         T = [a b 0; b c 0; 0 0 1];
%         C = [1 0 0; 0 1 0; 0 0 -1]; %unit circle
%         C_ = inv(T)'*C*inv(T)
