function M = topicMontage(DB,imageNdx,segmentNdx,HOMEIMAGES,HOMENCUTS,HOMEMERGE)
% M = topicMontage(DB,imageNdx,segmentNdx,HOMEIMAGES,HOMENCUTS,HOMEMERGE)
%
% Create a montage of the top segments belonging to a topic.
%
% Inputs:
% DB - LabelMe DB structure.
% imageNdx - Indices to the images containing the top segments.
% segmentNdx - Indices to the "merge" structure corresponding to the top
%   segments.
% HOMEIMAGES - Path to the images.
% HOMENCUTS - Path to the segmentation outputs.
% HOMEMERGE - Path to the words/segment merge outputs.
%
% Outputs:
% M - Montage of the top segments.

Nsegments = length(imageNdx);
mont_m = floor(sqrt(Nsegments));
mont_n = ceil(Nsegments/mont_m);
im_scale = 0.4; %% works for LabelMe and MSRa
%im_scale = 0.3; %% works for Caltech 5
bg_colors = [0.34 0.34 1];

a = 0;
for i = 1:Nsegments
  j = imageNdx(i);
  k = segmentNdx(i);

  img = double(LMimread(DB,j,HOMEIMAGES))/255;
  if size(img,3)~=3
    img(:,:,2) = img(:,:,1);
    img(:,:,3) = img(:,:,1);
  end
  load(fullfile(HOMENCUTS,DB(j).annotation.folder,[DB(j).annotation.filename '.mat']));
  load(fullfile(HOMEMERGE,DB(j).annotation.folder,[DB(j).annotation.filename '.mat']));
  
  m = find(Segs(:,:,merge(k).segmentationNdx)~=merge(k).segmentNdx);
  img(m) = bg_colors(1);
  img(m+size(img,1)*size(img,2)) = bg_colors(2);
  img(m+2*size(img,1)*size(img,2)) = bg_colors(3);
  a = a+1;
  allImg{a} = imresize(img,im_scale,'bicubic');
end

M = make_montage(allImg,mont_m,mont_n,bg_colors);

function im = make_montage(ims,m,n,fill_in_color)

  if nargin < 4
    fill_in_color = [0 0 0];
  end

  ims = ims(:);
  if length(ims) < m*n
    diff = m*n-length(ims);
    ims = [ims; cell(diff,1)];
  end
  ims = reshape(ims,n,m)';
  max_m = zeros(1,m);
  max_n = zeros(1,n);
  
  for nn = 1:n
    for mm = 1:m
      if size(ims{mm,nn},1) > max_m(mm)
        max_m(mm) = size(ims{mm,nn},1);
      end
      if size(ims{mm,nn},2) > max_n(nn)
        max_n(nn) = size(ims{mm,nn},2);
      end
    end
  end

  im = zeros(sum(max_m),sum(max_n),3);
  im(:,:,1) = fill_in_color(1);
  im(:,:,2) = fill_in_color(2);
  im(:,:,3) = fill_in_color(3);
  base_m = [1 cumsum(max_m(1:end-1))+1];
  base_n = [1 cumsum(max_n(1:end-1))+1];
  
  for nn = 1:n
    for mm = 1:m
      if size(ims{mm,nn},1) > 0
        im(base_m(mm):base_m(mm)+size(ims{mm,nn},1)-1, ...
           base_n(nn):base_n(nn)+size(ims{mm,nn},2)-1,:) = ...
            ims{mm,nn};
      end
    end
  end
